#!/usr/bin/env python3
# import sys
# sys.path.insert(1,'/home/jingyan/Documents/summer_intern_lemur/roco_electrical/dsn_python')

import dsnwritier
import csv
class brd_design():
    def __init__(self,boundary_inx,libpath,netlist_csv,savename='paperbot_ee',dwgfile='dwg_for_autorouter.dxf'):
        print('generating board desng file (dsn) ......')
        self.dwgfile=dwgfile
        self.boundary_inx=boundary_inx
        self.libpath=libpath
        self.savename=savename

        self.module_list=[
            ['mpu-9250.kicad_mod','J1',[115000,91000],'front',270],
            ['ESP12F-Devkit-V3.kicad_mod','U1',[104000,46100],'front',90],
            ['touch_sensor.kicad_mod','T1',[87000,90000],'front',0],
            ['LED_D3.0mm.kicad_mod','L1',[67000,41000],'front',90],
            ['Resistor.kicad_mod','R1',[72000,41000],'front',270],
            
        ]
        self.netlist=[
            ['3v3',['U1-16','J1-1','T1-2']],
            ['GND',['U1-24','J1-2','L1-2','T1-3']],
            ['NET1',['T1-1','R1-1']],
            ['NET2',['R1-2','L1-1']],
            ['SCL',['J1-3','U1-22']],
            ['SDA',['J1-4','U1-23']]
        ]
        # self.netlist=[
        #     ['3v3',['U1-16','J1-1','T1-2']],
        #     ['GND',['U1-24','J1-2','L1-2','T1-3']],
 
        # ]
        self.netclass_list=[
            ['default',['3v3','GND','NET1','NET2','SCL','SDA'],'',2000,500]
        ]
        # self.module_list,self.netlist,self.netclass_list=self.read_from_csv(netlist_csv)
        
        self.brd_general()
        self.boundary,self.keepout=self.load_drawing()
        self.image,self.padstack=self.load_lib()
        self.placement=self.place_modules()
        self.net,self.netclass=self.create_net()

        self.write_dsn()
        self.get_pin_loc()

    def brd_general(self):
        self.brd=dsnwritier.Dsn()
        self.layers=[
            dsnwritier.Layer('F.Cu')
            # dsnwritier.Layer('B.Cu'),
            # dsnwritier.Layer('F.Mask'),
            # dsnwritier.Layer('B.Mask')
        ]
        
        self.parsers= dsnwritier.Parser()
        
        self.rule=dsnwritier.Rule()
        clearance=[
            dsnwritier.Clearance(1000.1),
            dsnwritier.Clearance(1000.1,'default_smd'),
            dsnwritier.Clearance(1000,'smd_smd')]
        self.rule.clearance=clearance


    def load_drawing(self):
        """
        create class: bourdry and keepout 
        from drawing file
        """
        dwg=(dsnwritier.load_drawing(self.dwgfile)).load_polygon()
        bdata=dwg.pop(self.boundary_inx)
        kdata=dwg
        keepout=[]
        for i in range(len(kdata)):
            keep_class=dsnwritier.Keepout(kdata[i])
            keepout.append(keep_class)
        boundary=dsnwritier.Boundary(bdata)
        return boundary,keepout

    def load_lib(self):
        """
        create class: footprint (image) and padstack
        from library path and modules
        """

        image=[]
        
        for i in range(len(self.module_list)):
            mod=self.module_list[i]
            img=dsnwritier.Footprint.from_file(self.libpath+mod[0],ref=mod[1])
            image.append(img)
            if i==0:  
                padstack=dsnwritier.Padstack.auto_detect(self.libpath+mod[0])
            else:
                padstack+=dsnwritier.Padstack.auto_detect(self.libpath+mod[0])
        
        return image,padstack
    def place_modules(self):
        placement=[]
        for mod in self.module_list:
            place=dsnwritier.Placement(name=mod[0],ref1=mod[1],at=mod[2],flip=mod[3],orientation=mod[4])
            placement.append(place)
        return placement
    def create_net(self):
        nets_list=[]
        netclass_list=[]
        for net in self.netlist:
            nets=dsnwritier.Net(net_name=net[0],conn_pins=net[1])
            nets_list.append(nets)
        for netclass in self.netclass_list:  
            netclasses=dsnwritier.NetClass(net_class_name=netclass[0],
                                            nets_name=netclass[1],
                                            via_name=netclass[2],
                                            width=netclass[3],
                                            clearance=netclass[4])
            netclass_list.append(netclasses)
        return nets_list,netclass_list
    def write_dsn(self):
        self.brd.parser=self.parsers
        self.brd.rule=self.rule
        self.brd.layers=self.layers
        self.brd.boundary=self.boundary
        self.brd.keepout=self.keepout
        self.brd.image=self.image
        self.brd.padstack=self.padstack
        self.brd.placement=self.placement
        self.brd.net=self.net
        self.brd.netclass=self.netclass
        self.brd.to_file(self.savename)
        print('dsn file generated.....')
    def get_pin_loc(self):
        from pykicad.module import Module as mod
        import numpy as np
        from math import sqrt
        self.pins_at=[]

        for img in self.module_list:
            module=mod.from_file(self.libpath+img[0])
            pin_at_arr=[]
            module_at=np.array([img[2][0],img[2][1]])/1000
            flip=1
            orientation=img[4]
            theta=orientation/180*np.pi
            if img[3]=='back':
                flip=-1
                orientation=img[4]
            for i in range(len(module.pads)):
                pad=module.pads[i]
                pin_at_relative=np.array(pad.at) ##mm
                x,y=pin_at_relative[0]*flip,-pin_at_relative[1]
                x_=x*np.cos(theta)-y*np.sin(theta)
                y_=y*np.cos(theta)+x*np.sin(theta)
                pin_at_relative=np.array([x_,y_])
                pin_at_abs=list(pin_at_relative+module_at)
                pin_at_arr.append(pin_at_abs)
            self.pins_at.append(pin_at_arr)
    
    def read_from_csv(self,csv_file):
        with open(csv_file, mode='r') as csv_file:
            csv_reader = csv.DictReader(csv_file, delimiter = ';')
            myDict = list(csv_reader)

        numOfRows = len(myDict)
        keys = myDict[0].keys()

        moduleCols = 4
        netCols = 2
        netclassCols = 5

        moduleRows = 0
        netRows = 0
        netclassRows = 0 

        moduleHeaders = ['Module name','Ref','Position','Orientation']
        netHeaders = ['Connection name','Pins']
        netclassHeaders = ['Netclass','Net connections','Via name','Width','Clearance']

        for x in range(numOfRows):
            if myDict[x].get('Module name') != '' and myDict[x].get('Module name') != 'None':
                moduleRows += 1

            if myDict[x].get('Connection name') != '' and myDict[x].get('Connection name') != None:
                netRows += 1

            if myDict[x].get('Netclass') != '' and myDict[x].get('Netclass') != None:
                netclassRows +=1


        moduleList = [[0 for i in range(moduleCols)] for j in range(moduleRows)]
        netList =  [[0 for i in range(netCols)] for j in range(netRows)]
        netclassList = [[0 for i in range(netclassCols)] for j in range(netclassRows)]

        for x in range(moduleRows):
                for y in range(moduleCols):
                    header = moduleHeaders[y]
                    moduleList[x][y] = myDict[x].get(header)

        for x in range(netRows):
                for y in range(netCols):
                    header = netHeaders[y]
                    netList[x][y] = myDict[x].get(header)

        for x in range(netclassRows):
                for y in range(netclassCols):
                    header = netclassHeaders[y]
                    netclassList[x][y] = myDict[x].get(header)

        return moduleList,netList,netclassList


# libpath='/home/jingyan/Documents/summer_intern_lemur/roco_electrical/libraries/kicad-ESP8266/ESP8266.pretty/'
# dwgfile='/home/jingyan/Documents/summer_intern_lemur/roco_electrical/dsn_line_test.dxf'
# a= brd_design(dwgfile,0,libpath,'paperbot_ee')       

        


        





